%% Clearing the workspace
clear all; 

%% Closing all figures
close all; 

%% Loading Data
load('PEC_cal_plates.mat')

%% Defining vectors and values 
interval = 20; % Value set to obtain 300 micro seconds sampling interval 
Time = time(1:interval:end); % The time vector 
D_noLO = data_noLO(:,1:interval:end); % Data with 0 mm lift-off
D_LO = data_LO(:,1:interval:end); % Data with 12 mm lift-off

%% Visualizing noise margin with air signals
figure; 
plot(Time*1000, D_noLO(1,:),'k'); grid on; hold on; 
plot(Time*1000, D_LO(1,:),'r--'); grid on; hold on; 
xlabel('Time / (ms)');
ylabel('V(t) / (V)');
legend('Air Signal 1', 'Air Signal 2');

%% Visualizing noise margin with air signals in logarithmic scale
figure; 
plot(Time*1000, log(D_noLO(1,:)),'k'); grid on; hold on; 
plot(Time*1000, log(D_LO(1,:)),'r--'); grid on; hold on; 
ylim([-7 3]); 
xlabel('Time / (ms)');
ylabel('ln[V(t)]');
legend('Air Signal 1', 'Air Signal 2');

%% Defining the function
Signal_Type = D_LO; % Select dataset, i.e., D_noLO or D_LO
Signal_Number = 3; % Provide a number from 1 to 5 
noise_margin = 18e-3; % Noise margin 

%% Providing the initial conditions and boundaries for model parameters
Intcon = [0.01; 1; 1; 1; 1; 1; 1]; % Initial condition 
LB = [-0.1; 0; 0; 0; 0; 0; 0]; % Lower boundary
UB = [0.1; 1e4; 1e4; 1e4; 1e4; 1e4; 1e4]; % Upper boundary

%% Defining optimization options
% lsqnonlin 
% trust-region-reflective algorithm works with or without gradient 
% Levenberg-Marquardt algorithm does not handle boundaries

options = optimoptions('lsqnonlin', 'Algorithm',...
    'trust-region-reflective', 'SpecifyObjectiveGradient', false);%, ...
    %'PlotFcn', {@optimplotx, @optimplotfval, ...
    %@optimplotfirstorderopt});

%options.Display = 'iter';  
options.TolX = 1e-8; 
options.TolFun = 1e-8; 
options.MaxFunctionEvaluations = 10000;
options.MaxIter = 10000; 

%% Running the optimization code
tic;  
t = cputime;
Signal = [Time(2:end); Signal_Type(Signal_Number,2:end)]; 
Sig = Signal(2,Signal(2,:) < max(Signal(2,:)) & ...
    Signal(2,:) > noise_margin);
Time = Signal(1,Signal(2,:) < max(Signal(2,:)) & ...
    Signal(2,:) > noise_margin);

fun = @(params)costfun(Sig, Time, params);

[params_out, fval, exitflag, output] = lsqnonlin(fun, Intcon, LB, UB, ...
    options);  

params_ref = [params_out(3) params_out(5) params_out(7)]; 
feature = min(params_ref);  
e = cputime-t
toc; 

params = params_out; 
y = params(1) + params(2)*exp(-params(3)*Time) + ...
    params(4)*exp(-params(5)*Time) + params(6)*exp(-params(7)*Time); 

figure; 
plot(Time*1000, log(Sig),'k'); grid on; hold on; 
plot(Time*1000, log(y),'r--'); 
legend('Experimental Signal', 'Fitted Model'); 
xlabel('Time (ms)'); ylabel('ln[V(t)]');
ylim([-5 4]);

feature