%% Clearing the workspace
clear all; 

%% Closing all figures
close all; 

%% Loading Data
load('PEC_cal_plates.mat')

%% Defining vectors and values 
interval = 20; % Value set to obtain 300 micro seconds sampling interval 
Time = time(1:interval:end); % The time vector 
D_noLO = data_noLO(:,1:interval:end); % Data with 0 mm lift-off
D_LO = data_LO(:,1:interval:end); % Data with 12 mm lift-off

%% Plotting Signals, i.e., V(t)
figure; 
for i = 2:size(data_LO,1) 
     plot(time(1:20:end)*1000, data_noLO(i,1:20:end),'k'); grid on; hold on; 
     plot(time(1:20:end)*1000, data_LO(i,1:20:end),'k--'); grid on; hold on; 
end
xlabel('Time / (ms)');
ylabel('V(t) / (V)');
legend('0 mm lift-off', '12 mm lift-off');

%% Plotting Signals in logarithmic scale, i.e., ln[V(t)] 
figure; 
for i = 2:size(data_LO,1) 
     plot(time(1:20:end)*1000, log(data_noLO(i,1:20:end)),'k'); grid on; hold on; 
     plot(time(1:20:end)*1000, log(data_LO(i,1:20:end)),'k--'); grid on; hold on; 
end
ylim([-6 3]); 
xlabel('Time / (ms)');
ylabel('ln[V(t)]');
legend('0 mm lift-off', '12 mm lift-off');

%% Defining the function
Signal_Type = D_LO; % Select dataset, i.e., D_noLO or D_LO
Signal_Number = 5; % Provide a number from 1 to 5 
noise_margin = 18e-3; % Noise margin 

lower_bound = -4; 
upper_bound = -2; 

%% Running the optimization code
tic;  
t = cputime;

Signal = [Time(2:end); log(Signal_Type(Signal_Number,2:end))]; 
Sig = Signal(2,Signal(2,:) < upper_bound & Signal(2,:) > lower_bound);
Time = Signal(1,Signal(2,:) < upper_bound & Signal(2,:) > lower_bound);

params_out = [Time' ones(length(Time), 1)]\Sig'; 
feature = -params_out(1);  

e = cputime-t
toc; 

params = params_out; 
y = params(1)*Time + params(2);

figure; 
plot(Time*1000, Sig,'k'); grid on; hold on; 
plot(Time*1000, y,'r--'); 
legend('Experimental Signal', 'Fitted Model'); 
xlabel('Time (ms)'); ylabel('ln[V(t)]');
%ylim([-5 4]);

feature